;****************************************************************************
; Program		: I2C PCF8574 I/O 03
; Description	: I2C PCF8574 8 bit I/O , Sub address 000
;				: DIP Switch Set as Input & Run LED as Output
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
;
; NX-51 V2 NOTE : P1 Function switch must be in PORT1 mode
;
; Filename		: E1801.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
SDA				BIT		P1.0	; SDA I2C Bus
SCL				BIT		P1.1	; SCL I2C Bus
DRIVER_LE		BIT		P1.4	; Driver Latch Enable (Active High : Level)

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG		EQU		02FH		; User FLAG
I2C_ACK		BIT		FLAG.0		; Define I2C Acknowledge as bit

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
I2C_ADDR		EQU		033H		; For keep I2C Address
I2C_DATA		EQU		034H		; For keep I2C Data
IO_DATA			EQU		035H		; For keep I2C 8 bit I/O Data
OUT_DATA		EQU		036H		; For keep I2C 4 bit Output LED Data

;----------------------------------------------------------------------------
; Define I2C Slave Address
;----------------------------------------------------------------------------
PCF8574_ID		EQU		01110000B ; PCF8574A:000 Slave Address

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			MOV		OUT_DATA,#00010001B	; Set default value to drive LED

LOOP:			MOV		A,OUT_DATA		; Get LED Value
				MOV		B,A				; Store in B
				RL		A				; Rotate Left ACC.
				MOV		OUT_DATA,A		; Store new state
    
				MOV		A,B				; Restore B
				ANL		A,#00001111B	; Get Lower 4 bit
				MOV		R1,A			; Store Output Value
				ORL		A,#11110000B	; Set Higher 4 bit to High (Set Input)
				MOV		IO_DATA,A		; Move to IO_DATA
				ACALL	PCF8574_WR		; Write PCF8574

				ACALL	PCF8574_RD		; Read PCF8574
				MOV		A,IO_DATA		; Get Input Value
				ANL		A,#11110000B	; Get only Higher 4 bit
				ADD		A,R1			; Add with Output Value 4 bit

				MOV		P0,A			; Move data to P0
				ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock

				ACALL	DELAY_100ms		; Delay
 
				AJMP	LOOP			; Jump to loop

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
DRIVE_LE_CLK:	SETB	DRIVER_LE		; Driver Latch Enable
				NOP						; NOP
				CLR		DRIVER_LE		; Driver Latch Disable
				NOP
				RET

;----------------------------------------------------------------------------
; I2C PCF8574 Read
;----------------------------------------------------------------------------
PCF8574_RD:		MOV		I2C_ADDR,#PCF8574_ID+1 ; Set PCF8574 as I2C Read Slave
				ACALL	I2C_SLAVE		; Connect Slave

				ACALL	I2C_DATA_RD		; Read Data from Slave
				MOV		IO_DATA,I2C_DATA	; Read Data to IO_DATA
				ACALL	I2C_NACK_BIT	; Send Not Acknowledge

				ACALL	I2C_STOP		; Send Stop Condition
				RET						; Return

;----------------------------------------------------------------------------
; I2C PCF8574 Write
;----------------------------------------------------------------------------
PCF8574_WR:		MOV		I2C_ADDR,#PCF8574_ID ; Set PCF8574 as I2C Write Slave
				ACALL	I2C_SLAVE		; Connect Slave

				MOV		I2C_DATA,IO_DATA	; Write IO_DATA to Slave
				ACALL	I2C_DATA_WR		; Write Data to Slave

				ACALL	I2C_STOP		; Send Stop Condition
				RET						; Return

;----------------------------------------------------------------------------
; I2C Data Write
; I/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_WR:	PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_DATA		; Get Data
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_WR_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_DATA_WR_1	; Do until 8 times
				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_DATA_WR_2	; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK. bit
I2C_DATA_WR_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Data Read
; O/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_RD:	PUSH	ACC				; Push ACC.
				CLR		A				; Clear ACC.
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_RD_1:	ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				MOV		C,SDA			; Get SDA to Carry Flag
				RLC		A				; Rotate ACC. to Left with Carry
				CLR		SCL				; Clear SCL
				DJNZ	R5,I2C_DATA_RD_1; Do until 8 times
				MOV		I2C_DATA,A		; Move Data to I2C_DATA
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Slave Connect
; I/P:		I2C_ADDR
; O/P Flag:	I2C_ACK
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_SLAVE:		PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_ADDR		; Get Slave Address
				ACALL	I2C_START		; Send Start Condition
				MOV		R5,#008			; Set loop 8 times
I2C_SLAVE_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_SLAVE_1	; Do until 8 times

				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_SLAVE_2 ; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK.
I2C_SLAVE_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Start Condition
;----------------------------------------------------------------------------
I2C_START:		JNB		SCL,I2C_START_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_START_1:	SETB	SDA				; Set SDA
				SETB	SCL				; Set SCL

				ACALL	I2C_DELAY		; Delay
				CLR		SDA				; Clear SDA during SCL set
				ACALL	I2C_DELAY		; Delay
				CLR		SCL				; Clear SCL
				RET						; Return

;----------------------------------------------------------------------------
; I2C Stop Condition
;----------------------------------------------------------------------------
I2C_STOP:		JNB		SCL,I2C_STOP_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_STOP_1:		CLR		SDA				; Clear SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				SETB	SDA				; Set SDA during SCL set
				RET						; Return

;----------------------------------------------------------------------------
; I2C Clock
;----------------------------------------------------------------------------
I2C_CLK:		ACALL	I2C_DELAY		; Pulse SCL
				SETB	SCL				;
				ACALL	I2C_DELAY		;
				CLR		SCL				;
				RET						; Return

;----------------------------------------------------------------------------
; I2C Acknowledge
;----------------------------------------------------------------------------
I2C_ACK_BIT:	CLR		SDA				; Clear SDA
				ACALL	I2C_DELAY		; Delay
				ACALL	I2C_CLK			; Pulse I2C Clock
				SETB	SDA
				RET						; Return

;----------------------------------------------------------------------------
; I2C Not Acknowledge
;----------------------------------------------------------------------------
I2C_NACK_BIT:	SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				ACALL	I2C_CLK			; Pulse I2C Clock
				SETB	SCL
				RET						; Return

;----------------------------------------------------------------------------
; Dummy Delay time I2C_DELAY, 100m
;----------------------------------------------------------------------------
I2C_DELAY:		MOV		R6,#00CH		; Each loop = 50 us
I2C_DELAY_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,I2C_DELAY_1
				RET

DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET